## Copyright (C) 2021 The Octave Project Developers
##
## This program is free software; you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation; either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, see <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {Function File} {@var{retval} =} eval_derivative (@var{FHANDLE},@var{k},@var{x})
## Eval @var{k}-th derivative of function handle @var{FHANDLE} at value @var{x}.
## @var{FHANDLE} accepts following values
##  
## @itemize
## @item @@exp
## @item @@sin @@cos
## @item @@sinh @@cosh
## @end itemize
## 
## @end deftypefn

function retval = eval_derivative (FHANDLE, k, x)
   fname = (functions(FHANDLE)).function;
   if (fname == 'exp')
     retval = exp(x);
   elseif (fname == 'sin')
     residue = mod(k,4);
     switch (residue)
       case 0
         retval = sin(x);
       case 1
         retval = cos(x);
       case 2
         retval = -sin(x);
       case 3
         retval = -cos(x); 
     endswitch
   elseif (fname == 'cos')
     residue = mod(k,4);
     switch (residue)
       case 0
         retval = cos(x);
       case 1
         retval = -sin(x);
       case 2
         retval = -cos(x);
       case 3
         retval = sin(x); 
     endswitch
   elseif (fname == 'sinh')
     retval = cosh(x);
   elseif (fname == 'cosh')
     retval = sinh(x);
   else
     error("eval_derivative: %d-th derivative of function %s can't be evaluated",
           k,fname);
   endif
endfunction
